// ============================================================================
//  $Id$
//  $Name$
// ============================================================================
#include "TGlobals.hh"
#include "TDataSection.hh"
#include "TDataSegment.hh"
#include "TDataElement.hh"
#include "TOutputObjectFile.hh"

TDataSection* create( Tint val );

int main( int argc, char** argv )
{
  if ( argc != 2 ) {
    Tcerr << "Usage: " << argv[ 0 ] << Tspace << "<nevent>" << Tendl;
    return( -1 );
  }
  const Tint nev = atoi( argv[ 1 ] );
  const Tstring filename = "test.obj";
  TOutputObjectFile* ofile = new TOutputObjectFile( filename );
  Tint datasize = 0;

  for ( Tint i = 0; i < nev; i ++ ) {

    TDataSection* record = create( i );
    ofile -> WriteData( record );
    record -> Print();
    datasize += record -> GetDataSize();

    delete record;
  }

  Tcout << Tendl;
  Tcout << Ttab << datasize << " byte data ---> " << filename << Tendl;
  Tcout << Tendl;

  delete ofile;
  return( 0 );
}

TDataSection* create( Tint val )
{
  static const Tint secid = 0x1100;
  static const Tint seg1id = 0x1110;
  static const Tint seg2id = 0x1120;
  static const Tint seg3id = 0x1130;

  TDataSection* sec = new TDataSection( secid );
  TDataSegment* seg1 = new TDataSegment( seg1id );
  TDataSegment* seg2 = new TDataSegment( seg2id );
  TDataSegment* seg3 = new TDataSegment( seg3id );


  for ( Tint i = 0; i < 8; i ++ ) {
    Tint data = i + i * i + 0x1234 * val;
    seg1 -> Add( new TDataElement( &data, tTypeInt, i ) );
  }


  Ttime_t time_now;
  Tint pasttime = (Tint)time( &time_now );
  struct tm* tm_now = localtime( &time_now );
  Tsize_t length = 64;
  Tchar* tmp = new Tchar[ length ];
  strftime( tmp, length,"%Y/%m/%d %H:%M:%S", tm_now );
  string tstr = tmp;
  delete [] tmp;
  seg2 -> Add( new TDataElement( &pasttime, tTypeInt, 0 ) );
  seg2 -> Add( new TDataElement( &tstr, tTypeString, 1 ) );


  for ( Tdouble i = 0.0; i < 16.0; i ++ ) {
    Tdouble data = sin( i * i * val + val );
    seg3 -> Add( new TDataElement( &data, tTypeDouble, (Tint)i ) );
  }

  sec -> Add( seg1 );
  sec -> Add( seg2 );
  sec -> Add( seg3 );

  return( sec );
}
