// ============================================================================
//  $Id$
//  $Name$
// ============================================================================
#include "TTerminalUserInterface.hh"
#include "TCommand.hh"

static const Tchar TasciiCtrlA = '\001';
static const Tchar TasciiCtrlB = '\002';
static const Tchar TasciiCtrlD = '\004';
static const Tchar TasciiCtrlE = '\005';
static const Tchar TasciiCtrlF = '\006';
static const Tchar TasciiCtrlK = '\013';
static const Tchar TasciiCtrlL = '\014';
static const Tchar TasciiCtrlN = '\016';
static const Tchar TasciiCtrlP = '\020';
static const Tchar TasciiCtrlY = '\031';
static const Tchar TasciiCtrlZ = '\032';
static const Tchar TasciiTAB = '\011';
static const Tchar TasciiBS = '\010';
static const Tchar TasciiDEL = '\177';
static const Tchar TasciiESC = '\033';

TTerminalUserInterface::TTerminalUserInterface( const Tstring& prompt )
  : theCommandHistoryIndex( 0 ),
    theCursorPosition( 0 ), thePrompt( prompt ), theAcceptString( "" ),
    theStringBuffer( "" ), theCommandBuffer( "" ), theCommandHistory()
{
  tcgetattr( 0, &theTerminal );
}

TTerminalUserInterface::~TTerminalUserInterface()
{;}

Tvoid TTerminalUserInterface::initializeCommandLine()
{
  theCommandHistoryIndex = (Tint)theCommandHistory.size();
  theCursorPosition = 0;
  theAcceptString.erase();
  theCommandBuffer.erase();
  Tcout << thePrompt << Tflush;

  return;
}

const Tstring& TTerminalUserInterface::readLine()
{
  setTerminalInputMode();
  initializeCommandLine();

  Tchar cc;

  do {
    Tcin.get( cc );

    switch ( cc ) {

      case TasciiCtrlA:
	moveCursorTop();
	break;

      case TasciiCtrlB:
	backwardCursor();
	break;

      case TasciiCtrlD:
	deleteCharacter();
	break;

      case TasciiCtrlE:
	moveCursorEnd();
	break;

      case TasciiCtrlF:
	forwardCursor();
	break;

      case TasciiCtrlK:
	cutCharacter();
	break;

      case TasciiCtrlL:
	clearLine();
	break;

      case TasciiCtrlN:
	nextCommand();
	break;

      case TasciiCtrlP:
	previousCommand();
	break;

      case TasciiCtrlY:
	pasteCharacter();
	break;

      case TasciiCtrlZ:
	suspendTerminal();
	break;

      case TasciiTAB:
	if ( isCursorTop() && theAcceptString.empty() ) {
	  Tcout << Tendl;
	  ExecuteCommand( "ls" );
	  Tcout << Tflush;
	  return( theAcceptString );
	} else if ( isCursorEnd() ) {
	  completeCommand();
	}
	break;

      case TasciiBS:
	backspaceCharacter();
	break;

      case TasciiDEL:
	deleteCharacter();
	break;

      default:
	break;
    }

    if ( cc == TasciiESC ) {
      Tcin.get( cc );
      if ( cc == '[' ) {
	Tcin.get( cc );
	switch ( cc ) {
	  case 'A':  // up
	    cc = 'P' - '@';
	    previousCommand();
	    break;
	  case 'B':  // down
	    cc = 'N' - '@';
	    nextCommand();
	    break;
	  case 'C':  // right
	    cc = 'F' - '@';
	    forwardCursor();
	    break;
	  case 'D':  // left
	    cc = 'B' - '@';
	    backwardCursor();
	    break;
	  default:
	    cc = 0;
	    break;
	}
      }
    }
    
    insertCharacter( cc );

  } while ( cc != Teol );

  
  if ( theAcceptString.size() > 0 ) {
    while ( theAcceptString.rfind( Tspace ) == theAcceptString.size() - 1 )
      theAcceptString.erase( theAcceptString.end() - 1 );

    static const Tstring format = "%a %b %d %H:%M:%S";
    static const Tsize_t max = 256;
    Ttime_t time_now;
    time( &time_now );
    struct tm* tm_now = localtime( &time_now );
    Tchar* s = new Tchar[ max ];
    strftime( s, max, format.c_str(), tm_now );
    Tstring timestr = s;
    delete [] s;

    theHistoryFileStream << theCommandHistory.size() << Ttab;
    theHistoryFileStream << timestr << Ttab;
    theHistoryFileStream << theAcceptString << Tendl;

    theCommandHistory.push_back( theAcceptString );
  }
  Tcout << Tendl;

  resetTerminal();

  return( theAcceptString );
}

Tvoid TTerminalUserInterface::insertCharacter( Tchar cc )
{
  if ( ! isprint( cc ) )
    return;

  Tcout << cc;
  for ( Tint i = theCursorPosition; i < (Tint)theAcceptString.size(); i ++ )
    Tcout << theAcceptString[ i ];
  for ( Tint i = theCursorPosition; i < (Tint)theAcceptString.size(); i ++ )
    Tcout << TasciiBS;
  Tcout << Tflush;
    
  if ( isCursorEnd() )
    theAcceptString += cc;
  else
    theAcceptString.insert( theAcceptString.begin() + theCursorPosition, cc );

  theCursorPosition ++;
  return;
}

Tvoid TTerminalUserInterface::backspaceCharacter()
{
  if ( isCursorTop() )
    return;

  if ( isCursorEnd() ) {
    Tcout << TasciiBS << Tspace << TasciiBS;
  } else {
    Tcout << TasciiBS;
    for ( Tint i = theCursorPosition; i < (Tint)theAcceptString.size(); i ++ )
      Tcout << theAcceptString[ i ];
    Tcout << Tspace;
    for ( Tint i = theCursorPosition; i < (Tint)( theAcceptString.size() + 1 ); i ++ )
      Tcout << TasciiBS;
  }
  Tcout << Tflush;
  theAcceptString.erase( theCursorPosition - 1, 1 );
  theCursorPosition --;
  return;
}

Tvoid TTerminalUserInterface::deleteCharacter()
{
  forwardCursor();
  backspaceCharacter();
  return;
}

Tvoid TTerminalUserInterface::clearLine()
{
  moveCursorTop();
  clearAfterCursor();
  return;
}

Tvoid TTerminalUserInterface::clearAfterCursor()
{
  if ( isCursorEnd() )
    return;
  for ( Tint i = theCursorPosition; i < (Tint)theAcceptString.size(); i ++ )
    Tcout << Tspace;
  for ( Tint i = (Tint)theAcceptString.size(); i > theCursorPosition; i -- )
    Tcout << TasciiBS;
  Tcout << Tflush;
  theAcceptString.erase( theCursorPosition, theAcceptString.size() - theCursorPosition );
  return;
}

Tvoid TTerminalUserInterface::forwardCursor()
{
  if ( isCursorEnd() )
    return;
  Tcout << theAcceptString[ (Tsize_t)( theCursorPosition ) ] << Tflush;
  theCursorPosition ++;
  return;
}

Tvoid TTerminalUserInterface::backwardCursor()
{
  if ( isCursorTop() )
    return;
  Tcout << TasciiBS << Tflush;
  theCursorPosition --;
  return;
}

Tvoid TTerminalUserInterface::moveCursorTop()
{
  while ( isCursorTop() == Tfalse )
    backwardCursor();
  return;
}

Tvoid TTerminalUserInterface::moveCursorEnd()
{
  while ( isCursorEnd() == Tfalse )
    forwardCursor();
  return;
}

Tvoid TTerminalUserInterface::cutCharacter()
{
  if ( !( theStringBuffer.empty() ) )
    theStringBuffer.erase();
  for ( Tint i = theCursorPosition; i < (Tint)theAcceptString.size(); i ++ )
    theStringBuffer += theAcceptString[ i ];
  Tint nback = (Tint)( theAcceptString.size() ) - theCursorPosition;
  moveCursorEnd();
  for ( Tint i = 0; i < nback; i ++ )
    backspaceCharacter();
  return;
}

Tvoid TTerminalUserInterface::pasteCharacter()
{
  if ( theStringBuffer.empty() )
    return;
  for ( int i = 0; i < (Tint)theStringBuffer.size(); i ++ )
    insertCharacter( theStringBuffer[ i ] );
  return;
}

Tvoid TTerminalUserInterface::nextCommand()
{
  if ( theCommandHistoryIndex > (Tint)theCommandHistory.size() - 1 )
    return;
  clearLine();
  Tstring com;
  if ( theCommandHistoryIndex == (Tint)theCommandHistory.size() - 1 )
    com = theCommandBuffer;
  else
    com = theCommandHistory[ theCommandHistoryIndex + 1 ];

  for ( int i = 0; i < (Tint)com.size(); i ++ )
    insertCharacter( com[ i ] );
  theCommandHistoryIndex ++;
  return;
}

Tvoid TTerminalUserInterface::previousCommand()
{
  if ( theCommandHistory.empty() || theCommandHistoryIndex == 0 )
    return;

  if ( theCommandHistoryIndex == (Tint)theCommandHistory.size() )
    theCommandBuffer = theAcceptString;

  clearLine();

  Tstring com = theCommandHistory[ theCommandHistoryIndex - 1 ];
  for ( int i = 0; i < (Tint)com.size(); i ++ )
    insertCharacter( com[ i ] );
  theCommandHistoryIndex --;
  return;
}

Tvoid TTerminalUserInterface::completeCommand()
{
  // ڡĤ֤Ǥʤ
  Tstring strbuf = theAcceptString;
  TstringList candidate;
  Tstring shortest;

  TCommand* com = 0;
  SetCommandIndex( 0 );
  while ( ( com = NextCommand() ) ) {
    Tstring name = com -> GetCommandName();
    if ( shortest.empty() || shortest.size() > name.size() )
      shortest = name;
    if ( name.find( strbuf ) == 0 )
      candidate.push_back( name );
  }


  if ( !( candidate.empty() ) ) {
    if ( candidate.size() == 1 ) {
      clearLine();
      theAcceptString = candidate[ 0 ];
      theAcceptString += Tspace;
      theCursorPosition = theAcceptString.size();
      Tcout << theAcceptString;
    } else if ( candidate.size() >= 2 ) {
      Tcout << Tendl;
      for ( Tint i = 0; i < (Tint)candidate.size(); i ++ ) {
	Tcout << candidate[ i ];
	if ( i != (Tint)( candidate.size() - 1 ) )
	  Tcout << Twspace;
      }
      Tcout << Tendl;
      theAcceptString = shortest;
      theCursorPosition = shortest.size();
      Tcout << thePrompt << theAcceptString;
    }
  }

  Tcout << Tflush;

  return;
}

Tvoid TTerminalUserInterface::suspendTerminal() const
{
  pid_t pid = getpid();
  if ( fork() == 0 )
    exit( kill( pid, SIGSTOP ) );
  else
    wait( 0 );
  return;
}

Tvoid TTerminalUserInterface::setTerminalInputMode()
{
  termios tiosbuf = theTerminal;
  tiosbuf.c_iflag |= IGNBRK;
  tiosbuf.c_iflag |= IGNPAR;
  tiosbuf.c_lflag &= ~ICANON;
  tiosbuf.c_lflag &= ~ECHO;
  tiosbuf.c_lflag &= ~ISIG;
  tiosbuf.c_cc[ VMIN ] = 1;
  tiosbuf.c_cc[ VTIME ] = 0;
  tcsetattr( 0, TCSANOW, &tiosbuf );

  return;
}

Tvoid TTerminalUserInterface::resetTerminal()
{
  tcsetattr( 0, TCSANOW, &theTerminal );
  return;
}

TstringList TTerminalUserInterface::getArguments( const Tstring& readline )
{
  size_t begin = 0;
  size_t end = 0;
  TstringList strlist;

  size_t lastpos = readline.rfind( Tspace ) + 1;

  do {
    begin = readline.find_first_not_of( Tspace, end );
    end = readline.find( Tspace, begin );
    if ( end > readline.size() )
      end = readline.size();
    strlist.push_back( readline.substr( begin, end - begin ) );
  } while ( begin != lastpos );

  return( strlist );
}

Tvoid TTerminalUserInterface::SessionStart()
{
  //static const TUlong msec = 50 * 1000;  // 50 msec
  while ( ( theAcceptString.c_str() != 0 ) ) {
    Tstring strbuf = readLine();
    if ( strbuf.empty() )
      continue;

    TstringList args = getArguments( strbuf );
    Tstring comname = args[ 0 ];
    args.erase( args.begin() );

    TCommand* command = 0;
    if ( ( command = FindCommand( comname ) ) ) {
      if ( fork() == 0 ) {
	command -> Execute( args );
	exit( 0 );
      } else {
	//usleep( msec );  // ȤꤢϤǡ
	wait( 0 );
	if ( comname == "exit" ) {
	  break;
	}
      }
      //RunManagerΥơ⸫ʤƤ
    } else if ( !( comname.empty() ) ) {
      Tcerr << comname << ": Command not found." << Tendl;
    } else {
      continue;
    }



  }

  return;
}
